
;**************************************************************************
;*                       parallel SRAM via I2C                            *
;**************************************************************************
;**************************************************************************
;*          v 1.07 - 14.11.2013                                           *
;**************************************************************************
;*          Hardw.: PIC16F724 used                                        *
;*          OSC.......: Internal osc. used     POWER.....:  5V I2C if     *
;**************************************************************************



 INCLUDE "P16F724.INC"


	__CONFIG	_CONFIG1, _DEBUG_OFF & _PLL_EN & _BORV_2_5 & _BOR_OFF & _CP_OFF & _MCLR_DIS & _PWRT_DIS & _WDT_OFF & _INTRC_OSC_NOCLKOUT
	__CONFIG	_CONFIG2, _VCAP_DIS


;==========================================================================
;       Variable Definition
;==========================================================================
TIMER1		EQU	H'20'		;delay routine
TIMER2		EQU	H'21'		; "	"	"
disp1		EQU	H'22'		;first digit
disp2		EQU	H'23'		;second digit
LED1		EQU	H'24'
LED2		EQU	H'25'
LED3		EQU	H'26'
LED4		EQU	H'27'

;interrupt
W_TEMP		EQU	H'28'
STATUS_TEMP	EQU	H'29'
PCLATH_TEMP	EQU	H'2A'
;
v_buff1		EQU	H'2B'
v_add_h		EQU	H'2C'
v_add_l		EQU	H'2D'
v_data		EQU	H'2E'
;==========================================================================

		ORG	0		;Reset vector address
		GOTO	RESET		;goto RESET routine when boot.

;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX


;----------------------------------------------------------------------------
;----------------------------------------------------------------------------
;
;	       **********************************
;              **  RESET :  main boot routine  **
;              **********************************

RESET		nop
PAGE0
;
		movlw	D'255'
		movwf	PORTC

		BSF	STATUS,RP0	;Switch to register bank 1
		MOVLW	B'01011111'	;Set PIC options (See datasheet).
		MOVWF	OPTION_REG	;Write the OPTION register.
		bcf	OPTION_REG,T0CS	;set-up TIMER0 as timer
		bsf	OPTION_REG,PSA	;prescaler NOT for TIMER0
		;
		CLRF	INTCON		;Disable interrupts


		MOVLW	B'10000000'	;PORTA --> A8 - A14
		MOVWF	TRISA

		MOVLW	B'11111111'	;PORTB --> D0 - D7
		MOVWF	TRISB

		MOVLW	B'00011111'	;PORTC --> control + I2C
		MOVWF	TRISC

		MOVLW	B'00000000'	;PORTD --> A0 - A7
		MOVWF	TRISD

		MOVLW	B'00001000'	;PORTE --> LEDs
		MOVWF	TRISE

		movlw	H'FF'
		movwf	WPUB		;enable pull-ups for all RB bits

		bsf	STATUS,RP0
		bsf	STATUS,RP1
		clrf	ANSELB
		clrf	ANSELA

		BCF	STATUS,RP0	;Switch Back to reg. Bank 0
		bcf	STATUS,RP1

; XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
; XXXXXXXXXX       initialization  XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
; XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
; ---------------------------------------------------------------------------
;    SRAM Setup
; ---------------------------------------------------------------------------

		movlw	D'255'
		movwf	PORTC

		movwf	PORTA			;set address to high address (in case an EPROM is used)
		movwf	PORTD			;set address to high address (in case an EPROM is used)

; ---------- I2C setup ------------------------------------------------------
		movlw	B'00110110'
		movwf	SSPCON			;See Page 186 of Datasheet
;set-up the slave address
		bsf	STATUS,RP0
		movlw	B'10100000'		;set the I2C slave address to EEPROM-like address: 1010 000
		movwf	SSPADD
		clrf	SSPSTAT
		bcf	STATUS,RP0

		bcf	PIR1,SSPIF
; ---------------------------------------------------------------------------


blink01		movlw	B'00000011'
		movwf	PORTE
		call	dr2
		movlw	B'00000001'
		movwf	PORTE
		call	dr2
		movlw	B'00000010'
		movwf	PORTE
		call	dr2

		clrf	PORTE
;---------------------------------------------------------------------------
;---------------------------------------------------------------------------
;---------------------------------------------------------------------------
;---------------------------------------------------------------------------
idle_main	nop
		movlw	B'00000000'
		movwf	PORTE
		btfss	PIR1,SSPIF
		goto	idle_main
;we received a matching I2C address
		movf	SSPBUF,0		;read the received address to clear the buffer
		movwf	v_buff1
		bcf	PIR1,SSPIF

		bsf	STATUS,RP0
		btfsc	SSPSTAT,D'005'		;Data = 1 / Address = 0
		goto	data_rx			;if we got data on this stage, it can be only word to be written to SRAM

		bcf	STATUS,RP0
		btfss	v_buff1,D'000'
		goto	addr_h			;I2C direction is "write"; we will expect the address at this stage
		goto	data_tx			;I2C direction is "read"; we will provide data

;now we expect the address - MSB --------------------
;
addr_h		nop
		btfss	PIR1,SSPIF
		goto	addr_h
		movf	SSPBUF,0
		movwf	v_add_h
		bcf	PIR1,SSPIF

;now we expect the address - LSB --------------------
;
addr_l		nop
		btfss	PIR1,SSPIF
		goto	addr_l
		movf	SSPBUF,0
		movwf	v_add_l
		bcf	PIR1,SSPIF
;
		goto	idle_main
; ------------------------------------------------------------------
; ------------------------------------------------------------------
; ------------------------------------------------------------------

;now we got the data --------------------
;
data_rx		nop
		bcf	STATUS,RP0
		movlw	B'00000001'
		movwf	PORTE
		movf	v_buff1,0
		movwf	v_data
		bcf	PIR1,SSPIF

;we have to write the data to the SRAM now ---
		bsf	STATUS,RP0
		movlw	B'00000000'
		movwf	TRISB
		bcf	STATUS,RP0

		movf	v_add_h,0
		movwf	PORTA
		movf	v_add_l,0
		movwf	PORTD
		movf	v_data,0
		movwf	PORTB

		bcf	PORTC,D'005'		;activate CS
		nop
		bcf	PORTC,D'007'		;acivate WE
		nop
		nop
		bsf	PORTC,D'007'
		bsf	PORTC,D'005'
		bsf	STATUS,RP0
		movlw	B'11111111'
		movwf	TRISB
		bcf	STATUS,RP0

;execute address autoincrement --------------
		incf	v_add_l,1
		btfsc	STATUS,Z
		incf	v_add_h,1
;--------------------------------------------
		goto	idle_main
;---------------------------------------------------------------------------
;---------------------------------------------------------------------------
;now we send the addressed byte to I2C bus
data_tx		nop
		movlw	B'00000010'
		movwf	PORTE
		bsf	STATUS,RP0
		movlw	B'11111111'
		movwf	TRISB
		bcf	STATUS,RP0

;we have to get the byte from SRAM now ----------
		movf	v_add_h,0
		movwf	PORTA
		movf	v_add_l,0
		movwf	PORTD
		bcf	PORTC,D'005'		;activate CS
		nop
		bcf	PORTC,D'006'		;acivate OE
		nop
		movf	PORTB,0
		bsf	PORTC,D'006'
		bsf	PORTC,D'005'
;start the I2C transmission
		movwf	SSPBUF
		bsf	SSPCON,CKP		;start the transmission

;wait until the transmission is finished
data_txw2	nop
		btfss	PIR1,SSPIF
		goto	data_txw2

;--------------------------------------------------------------------------------
;wait for ACK or NOT ACK, but how? ...
;we try it with STOP bit detection
		btfss	SSPSTAT,D'004'
		goto	next_one


		bcf	PIR1,SSPIF


		goto	idle_main		;communication has been terminated
;--------------------------------------------------------------------------------
;just send next byte
;execute address autoincrement --------------
next_one	bcf	PIR1,SSPIF

		incf	v_add_l,1
		btfsc	STATUS,Z
		incf	v_add_h,1
;
		goto	data_tx

;---------------------------------------------------------------------------
;---------------------------------------------------------------------------
;---------------------------------------------------------------------------

;-------------------------------------------------------------------------
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX

;		*********************************************
;		*               delay routine               *
;		*********************************************

delay_routine	MOVLW   D'255'
		MOVWF   TIMER2
DEL_LOOP1	MOVLW   D'255'
		MOVWF   TIMER1

DEL_LOOP2	DECFSZ  TIMER1,F
		GOTO    DEL_LOOP2
		DECFSZ  TIMER2,F
		GOTO    DEL_LOOP1
		return

dr2		call	delay_routine
		call	delay_routine
		return

;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;-------------------------------------------------------------------------------
;-------------------------------------------------------------------------------



		END